from typing import List
import warnings

from pydantic import BaseModel, validator
from . import config


class CheckClintParam(BaseModel):
    user: str = "root"
    password: str = "123123"
    host: List[str]


class CheckCreateIndexParam(BaseModel):
    index_name: str
    dimension: int
    index_type: str = "hnsw"
    metric_type: str = "euclidean"
    replicas: int = 3
    index_config: dict = None
    metadata_config: dict = None
    partition_rule: dict = None
    operand: list = None
    auto_id: bool = True
    start_id: int = 1

    @validator("index_type", always=True)
    def check_index_type(cls, value):
        if value not in config.index_config.keys():
            raise Exception(f"index_type  must in {list(config.index_config.keys())}")
        return value
    
    @validator("metric_type", always=True)
    def check_metric_type(cls, value):
        if value not in config.metric_type.keys():
            raise Exception(f"metric_type  must in {list(config.metric_type.keys())}")
        value = config.metric_type[value]
        return value
    
    @validator("replicas", always=True)
    def check_replicas(cls, value):
        if value < 0:
            raise Exception(f"{value} must >= 0")
        return value
    
    @validator("index_config", always=True)
    def check_index_config(cls, value, values):
        index_type = values.get("index_type")
        dimension = values.get("dimension")
        metric_type = values.get("metric_type")
        vector_index_parameter = config.index_config[index_type]
        
        if value is not None:
            index_keys = vector_index_parameter[list(vector_index_parameter.keys())[0]].keys()
            for key, v in value.items():
                if key in index_keys:
                    vector_index_parameter[list(vector_index_parameter.keys())[0]][key] = v
                    if key == "efConstruction" and (v > 500 or v < 100):
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        warnings.warn(f"efConstruction: {v} suggestion in 100-500")
                    if key == "maxElements" and (v > 1000000000 or v < 50000):
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        warnings.warn(f"maxElements:{v} suggestion in 50000-1000000000")
                    if key == "nlinks" and (v > 64 or v < 16):
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        warnings.warn(f"nlinks:{v} suggestion in 16-64")
                else:
                    warnings.warn(f"index_config {key} not in {index_type}")
        
        vector_index_parameter[list(vector_index_parameter.keys())[0]]["dimension"] = dimension
        vector_index_parameter[list(vector_index_parameter.keys())[0]]["metricType"] = metric_type
        return vector_index_parameter
    
    @validator("partition_rule", always=True)
    def check_partition_rule(cls, value):
        value = {} if value is None else value
        return value
    

class CheckVectorAddParam(BaseModel):
    index_name: str
    vectors: List[list]
    datas: List[dict] = None
    ids: List[int] = None

    @validator("datas", always=True)
    def check_datas(cls, value, values):
        value = [{}] * len(values.get("vectors")) if value is None else value
        return value

    @validator("ids", always=True)
    def check_ids(cls, value, values):
        if value is None:
            assert len(values.get("datas")) == len(values.get("vectors")), \
                f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} is not equal"
        else:
            assert len(values.get('datas')) == len(values.get('vectors')) == len(value), \
                f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} ids:{len(value)} is not equal"
        return value


class CheckVectorScanParam(BaseModel):
    index_name: str
    start_id: int
    max_count: int = 1000
    is_reverse: bool = False
    with_scalar_data: bool = True
    with_table_data: bool = True
    without_vector_data: bool = False
    filter_scalar: list = None
    
    @validator("*", always=True)
    def check_input(cls, value, field):
        if field.name == "start_id":
            if not value > 0:
                raise Exception("start_id must > 0")   
        if field.name == "max_count":
            if not value > 0:
                raise Exception("max_count must > 0")
        if field.name == "is_reverse" or field.name == "with_scalar_data" or field.name == "with_table_data" or \
                field.name == "without_vector_data":
            value = "true" if value else "false"
        if field.name == "filter_scalar":
            if value is None:
                value = []

        return value


class CheckVectorSearchParam(BaseModel):
    index_name: str
    xq: list
    top_k: int = 10
    search_params: dict = None

    @validator("xq", always=True)
    def check_xq(cls, value):
        if not isinstance(value[0], list):
            value = [value]
        return value
    
    @validator("search_params", always=True)
    def check_search_params(cls, search_params, values):
        scalar_data = {}
        use_scalar_filter = "false"
        if search_params is not None and "meta_expr" in search_params.keys():
            if search_params["meta_expr"] is not None:
                use_scalar_filter = "true"
                scalar_data = dict(
                    (key, {"fieldType": "STRING", "fields": [{"data": value}]})
                    for key, value in search_params["meta_expr"].items()
                )
        
        ef_search = 32 if search_params is None else search_params.get("efSearch", 32)
        assert ef_search >= 0, f"efSearch must >= 0, but get {ef_search}"

        payload = {
            "parameter": {
                "search": {
                    "hnswParam": {
                        "efSearch": ef_search
                        },
                    "flat":
                        {
                        "parallelOnQueries": 0 if search_params is None else search_params.get("parallelOnQueries", 0)
                        }
                    },
                "selectedKeys": [],
                "topN": values.get("top_k"),
                "withScalarData": "true" if search_params is None else search_params.get("withScalarData", "true"),
                "withoutVectorData": "false" if search_params is None else search_params.get("withoutVectorData",
                                                                                             "false"),
                "useScalarFilter": use_scalar_filter
                },
            "vectors": [{
                "scalarData": scalar_data,
                "vector": {
                    "binaryValues": [],
                    "dimension": len(xq),
                    "floatValues": xq,
                    "valueType": "FLOAT"
                            }
                        } for xq in values.get("xq")]
                }
        return payload


class CheckVectorGetParam(BaseModel):
    index_name: str
    ids: List[int]
    scalar: bool = True
    vector: bool = True


class CheckVectorDeleteParam(BaseModel):
    index_name: str
    ids: List[int]



metric_type = {
    "euclidean": "METRIC_TYPE_L2",
    "dotproduct": "METRIC_TYPE_INNER_PRODUCT"
}


# proto
index_config = {
    "flat": {
                "flatParam": {
                    "dimension": None,
                    "metricType": None
                },
                "vectorIndexType": "VECTOR_INDEX_TYPE_FLAT"
            },
    "ivf_flat": {
                "ivfFlatParam": {
                    "dimension": None,
                    "metricType": None,
                    "ncentroids": 256
                },
                "vectorIndexType": "VECTOR_INDEX_TYPE_IVF_FLAT"
            },
    "ivf_pq": {
                "ivfPqParam": {
                    "bucketInitSize": 0,
                    "bucketMaxSize": 0,
                    "dimension": None,
                    "metricType": None,
                    "ncentroids": 256,
                    "nsubvector": 8
                },
                "vectorIndexType": "VECTOR_INDEX_TYPE_IVF_PQ"
            },
    "hnsw": {
                "hnswParam": {
                            "dimension": None,
                            "efConstruction": 200,
                            "maxElements": 50000,
                            "metricType": None,
                            "nlinks": 32
                        },
                "vectorIndexType": "VECTOR_INDEX_TYPE_HNSW"
            },
    "diskann": {
                "diskAnnParam": {
                    "dimension": None,
                    "metricType": None
                },
                "vectorIndexType": "VECTOR_INDEX_TYPE_DISKANN"
            }
}



#!/usr/bin/python3
import json
import requests

from .dingo_param import CheckClintParam, CheckCreateIndexParam, CheckVectorAddParam, CheckVectorSearchParam, \
    CheckVectorGetParam, CheckVectorDeleteParam, CheckVectorScanParam


class DingoDB:
    headers = {"Content-Type": "application/json"}
    requestProto = "http://"
    indexApi = "/index/api/dingo/"
    vectorApi = "/vector/api/dingo/"
    
    def __init__(self, user: str, password: str, host: list) -> None:
        """
        __init__ init DingoDB

        Args:
            user (str): DingoDB user
            password (str): DingoDB password
            host (list): DingoDB host:port
        """
        params = CheckClintParam(user=user, password=password, host=host)
        self.user = params.user
        self.password = params.password
        self.host = params.host
        self.session = requests.Session()
    
    def __del__(self):
        self.session.close()
        
    def close(self):
        self.session.close()

    def describe_index_info(self, index_name: str) -> dict:
        """
        describe_index_info index info

        Args:
            index_name (str): the name the index

        Raises:
            RuntimeError: return error

        Returns:
            dict: index info
        """
        res = self.session.get(f"{self.requestProto}{self.host[0]}{self.indexApi}{index_name}", headers=self.headers)
        
        if res.status_code == 200:
            return res.json()
        else:
            raise RuntimeError(res.json())

    def create_index(self,  index_name: str, dimension: int, index_type: str = "hnsw", metric_type: str = "euclidean",
                     replicas: int = 3, index_config: dict = None, metadata_config: dict = None,
                     partition_rule: dict = None, operand: list = None, auto_id: bool = True,
                     start_id: int = 1) -> bool:
        """
        create_index create index

        Args:
            index_name (str): the name of index
            dimension (int): dimension of vector
            index_type (str, optional): index type, one of {"flat", "hnsw"}. Defaults to "hnsw".
            metric_type (str, optional): metric type, one of {"dotproduct", "euclidean"}. Defaults to "euclidean".
            replicas (int, optional): dingoDB store replicas. Defaults to 3.
            index_config (dict, optional): Advanced configuration options for the index. Defaults to None.
            metadata_config (dict, optional): metadata. Defaults to None.
            partition_rule (dict, optional): partition rule. Defaults to None.
            operand (list, optional): operand. Defaults to None.
            auto_id (bool, optional): isAutoIncrement or not isAutoIncrement. Defaults to True.
            start_id (int, optional): autoIncrement start id. Defaults to 1.

        Raises:
            RuntimeError: return error

        Returns:
            bool: create table status
        """
        params = CheckCreateIndexParam(index_name=index_name, dimension=dimension, index_type=index_type,
                                       metric_type=metric_type, replicas=replicas, index_config=index_config,
                                       metadata_config=metadata_config, partition_rule=partition_rule, operand=operand,
                                       auto_id=auto_id, start_id=start_id)

        if params.partition_rule == {} and operand is not None and len(operand) != 0:
            details = []
            for item in operand:
                details.append(
                    {
                        "operand": [item],
                        "operator": "",
                        "partName": ""
                    }
                )
            partition_rule = {
                "cols": [],
                "details": details,
                "funcName": ""
            }

        index_definition = {
            "autoIncrement": params.start_id,
            "isAutoIncrement": "true" if params.auto_id else "false",
            "indexParameter": {
                "indexType": "INDEX_TYPE_VECTOR",
                "vectorIndexParameter": params.index_config
            },
            "name": params.index_name,
            "indexPartition": partition_rule if partition_rule is not None else params.partition_rule,
            "replica": params.replicas,
            "version": 0
        }
        res = self.session.post(f"{self.requestProto}{self.host[0]}{self.indexApi}", headers=self.headers,
                                data=json.dumps(index_definition))
        if res.status_code == 200:
            return True
        else:
            raise RuntimeError(res.json())

    def update_index_max_element(self, index_name: str, max_element: int) -> bool:
        """
        update_index_max_element change index max element

        only for hnsw

        Args:
            index_name (str): the name of index
            max_element (int): max element value

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        
        res = self.session.put(f"{self.requestProto}{self.host[0]}{self.indexApi}{index_name}/{max_element}")
        if res.status_code == 200:
            return True
        else:
            raise RuntimeError(res.json())
    
    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        res = self.session.delete(f"{self.requestProto}{self.host[0]}{self.indexApi}{index_name}")
        if res.status_code == 200:
            return True
        else:
            raise RuntimeError(res.json())

    def vector_add(self, index_name: str, datas: list, vectors: list, ids: list = None) -> list:
        """
        vector_add add vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list 
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: add vector info in dingoDB
        """
        params = CheckVectorAddParam(index_name=index_name, datas=datas, vectors=vectors, ids=ids)

        records = []
        for i, v in enumerate(params.vectors):
            scalar_data = dict(
                    (key, {"fieldType": "STRING", "fields": [{"data": value}]})
                    for key, value in params.datas[i].items()
                )
            vector = {
                    "binaryValues": [],
                    "dimension": len(v),
                    "floatValues": v,
                    "valueType": "FLOAT"
            }
            record = {
                    "scalarData": scalar_data,
                    "vector": vector
                }
            if ids is not None:
                record.update({"id": params.ids[i]})
            
            records.append(record)
        res = self.session.put(f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}",
                               headers=self.headers, data=json.dumps(records))
        if res.status_code == 200:
            records = res.json()
            return records
        else:
            raise RuntimeError(res.json())
    
    def vector_count(self, index_name: str):
        """
        vector_count count in index

        Args:
            index_name (str): the name of in index
        
        Returns:
            int: count num
        """
        res = self.session.get(f"{self.requestProto}{self.host[0]}{self.vectorApi}{index_name}", headers=self.headers)
        if res.status_code == 200:
            records = res.json()
            return records.get("currentCount") - records.get("deletedCount")
        else:
            raise RuntimeError(res.json())
        
    def vector_scan(self, index_name: str, start_id: int, max_count: int = 1000, is_reverse: bool = False,
                    with_scalar_data: bool = True, with_table_data: bool = True, without_vector_data: bool = False,
                    filter_scalar: list = None) -> list:
        """
        vector_scan scan with start_id

        Args:
            index_name (str): the name of in index
            start_id (int): start id
            max_count (int, optional): max scan count. Defaults to 1000.
            is_reverse (bool, optional): whether or not reverse. Defaults to False.
            with_scalar_data (bool, optional): whether  with scalar info. Defaults to True.
            with_table_data (bool, optional): whether  with table info. Defaults to True.
            without_vector_data (bool, optional): whether with vector info. Defaults to False.
            filter_scalar (list, optional): scalar filter filed. Defaults to [].

        Raises:
            RuntimeError: return error

        Returns:
            list:  scan info list
        """
        params = CheckVectorScanParam(index_name=index_name, start_id=start_id, max_count=max_count,
                                      is_reverse=is_reverse, with_scalar_data=with_scalar_data,
                                      with_table_data=with_table_data, without_vector_data=without_vector_data,
                                      filter_scalar=filter_scalar)
        payload = {
            "isReverseScan": params.is_reverse,
            "maxScanCount": params.max_count,
            "selectedKeys": params.filter_scalar,
            "startId": params.start_id,
            "withScalarData": params.with_scalar_data,
            "withTableData": params.with_table_data,
            "withoutVectorData": params.without_vector_data
        }
        res = self.session.post(f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}/scan",
                                headers=self.headers, data=json.dumps(payload))
        if res.status_code == 200:
            records = res.json()
            return records
        else:
            raise RuntimeError(res.json())
        
    def get_index(self):
        """
        get_index get all index

        Raises:
            RuntimeError: return error

        Returns:
            list: all index list 
        """

        res = self.session.get(f"{self.requestProto}{self.host[0]}{self.indexApi}", headers=self.headers)
        if res.status_code == 200:
            indics = res.json()
            return indics
        else:
            raise RuntimeError(res.json())

    def get_max_index_row(self, index_name: str):
        """
        get_max_index_row get max id in index

        Args:
            index_name (str): the name of in index 

        Raises:
            RuntimeError: return error

        Returns:
            int: max id value
        """
        payload = {"isGetMin": "false"}

        res = self.session.get(f"{self.requestProto}{self.host[0]}{self.vectorApi}{index_name}/id",
                               params=payload, headers=self.headers)
        if res.status_code == 200:
            max_id = res.json()
            return max_id
        else:
            raise RuntimeError(res.json())

    def vector_search(self, index_name: str, xq: list, top_k: int = 10, search_params: dict = None) -> dict:
        """
        vector_search search vector

        Args:
            index_name (str): the name of the index 
            xq (list): query vector, List[float] or List[List[float]]
            top_k (int, optional): top k search. Defaults to 10.
            search_params (dict, optional): search params for index. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: search results
        """
        params = CheckVectorSearchParam(index_name=index_name, xq=xq, top_k=top_k, search_params=search_params)
        
        res = self.session.post(f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}",
                                headers=self.headers, data=json.dumps(params.search_params))
        
        if res.status_code == 200:
            records = res.json()
            return records
        else:
            raise RuntimeError(res.json())
    
    def vector_get(self, index_name: str, ids: list, scalar: bool = True, vector: bool = True) -> list:
        """
        vector_get query vector

        Args:
            index_name (str): the name of the index
            ids (list): query id list 
            scalar (bool, optional): res with or without scalar. Defaults to True.
            vector (bool, optional): res with or without vector. Defaults to True.

        Raises:
            RuntimeError: _description_

        Returns:
            list: _description_
        """
        params = CheckVectorGetParam(index_name=index_name, ids=ids, scalar=scalar, vector=vector)
        payload = {
            "ids": params.ids,
            "keys": [],
            "withScalarData": "true" if scalar else "false",
            "withoutVectorData": "false" if vector else "true"
        }
        
        res = self.session.post(f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}/get",
                                headers=self.headers, data=json.dumps(payload))
        
        if res.status_code == 200:
            records = res.json()
            return records
        else:
            raise RuntimeError(res.json())

    def vector_delete(self, index_name: str, ids: list):
        """
        vector_delete delete vector with ids

        Args:
            index_name (str): the name of the index
            ids (list): id list 

        Raises:
            RuntimeError: return error

        Returns:
            list : [True, False, ...]
        """
        params = CheckVectorDeleteParam(index_name=index_name, ids=ids)
        
        res = self.session.delete(f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}",
                                  headers=self.headers, data=json.dumps(params.ids))
        
        if res.status_code == 200:
            return res.json()
        else:
            raise RuntimeError(res.json())



#!/usr/bin/python3

import dingodb
import numpy as np

x = dingodb.DingoDB("user", "password", ["172.20.3.20:13000"])


# index_name
# dimension
# index_type "flat"/"ivf_flat"/"ivf_pq"/"hnsw"/"diskann"
# metric_type "euclidean"/"product"/None
# replicas
# index_config = {"efConstruction":n, "maxElements":n, "nlinks":n}
# metadata_config = {}
# partition_rule = {}
# operand = []
b1 = x.create_index("test", 6, "hnsw", "euclidean", 3, {}, {}, {}, [5,10,15,20])
print(b1)
# b1 = True
# RuntimeError

# index_name
b2 = x.describe_index_info("test")
print(b2)
# b2 = {'name': 'test', 'version': 0, 'replica': 3, 'isAutoIncrement': True, 'autoIncrement': 1, 'indexParameter': {'indexType': 'INDEX_TYPE_VECTOR', 'vectorIndexParameter': {'vectorIndexType': 'VECTOR_INDEX_TYPE_HNSW', 'flatParam': None, 'ivfFlatParam': None, 'ivfPqParam': None, 'hnswParam': {'dimension': 6, 'metricType': 'METRIC_TYPE_L2', 'efConstruction': 200, 'maxElements': 50000, 'nlinks': 32}, 'diskAnnParam': None}}, 'indexPartition': {'funcName': 'RANGE', 'cols': [], 'details': [{'partName': '', 'operator': '', 'operand': [5]}, {'partName': '', 'operator': '', 'operand': [10]}, {'partName': '', 'operator': '', 'operand': [15]}, {'partName': '', 'operator': '', 'operand': [20]}]}}
# RuntimeError

# 构建训练数据和查询数据
d = 6                           # dimension
nb = 4                      # database size
np.random.seed(1234)             # make reproducible
xb = np.random.random((nb, d)).astype('float32')
xb[:, 0] += np.arange(nb) / 1000.

ids = [1, 2, 3, 4]
datas = [{"a1": "b1"}, {"a2": "b2"}, {"a3": "b3"}, {"a4": "b4"}]
vectors = xb.tolist()
# vectors = [[321.213,3213.22,1,0,32.3,0.5],[3212.213,32513.22,1,50,32.3,0.5],[321.26413,32143.22,14536,0,32.345,0.5],[334534321.213454,3213453453.22,1,0,3265.3,0.5]]
# index_name
# ids = [id1, id2, id3]
# datas = [{data1key:data1value}, {data2key:data2value}, {data3key:data3value}]
# vectors = [[float1_1, float1_2, float1_3], [float2_1, float2_2, float2_3], [float3_1, float3_2, float3_3]]
b3 = x.vector_add("test", datas, vectors, ids)
print(b3)
# b3 = [{'id': 1, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.19151945, 0.62210876, 0.43772775, 0.7853586, 0.77997583, 0.2725926], 'binaryValues': []}, 'scalarData': {'a1': {'fieldType': 'STRING', 'fields': [{'data': 'b1'}]}}}, {'id': 2, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.27746424, 0.8018722, 0.95813936, 0.87593263, 0.35781726, 0.5009951], 'binaryValues': []}, 'scalarData': {'a2': {'fieldType': 'STRING', 'fields': [{'data': 'b2'}]}}}, {'id': 3, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.6854629, 0.71270204, 0.37025076, 0.5611962, 0.50308317, 0.013768449], 'binaryValues': []}, 'scalarData': {'a3': {'fieldType': 'STRING', 'fields': [{'data': 'b3'}]}}}, {'id': 4, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.77582663, 0.8826412, 0.364886, 0.6153962, 0.07538124, 0.368824], 'binaryValues': []}, 'scalarData': {'a4': {'fieldType': 'STRING', 'fields': [{'data': 'b4'}]}}}]
# RuntimeError

# index_name
# xq = [float1, float2, float3]
# topk
# search_paras = []
b4 = x.vector_search("test", vectors[0])
print(b4)
# b4 = {'vectorWithDistances': [{'id': 1, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.19151945, 0.62210876, 0.43772775, 0.7853586, 0.77997583, 0.2725926], 'binaryValues': []}, 'scalarData': {'a1': {'fieldType': 'STRING', 'fields': [{'data': 'b1'}]}}, 'distance': 0.0}, {'id': 3, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.6854629, 0.71270204, 0.37025076, 0.5611962, 0.50308317, 0.013768449], 'binaryValues': []}, 'scalarData': {'a3': {'fieldType': 'STRING', 'fields': [{'data': 'b3'}]}}, 'distance': 0.4506487}, {'id': 2, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.27746424, 0.8018722, 0.95813936, 0.87593263, 0.35781726, 0.5009951], 'binaryValues': []}, 'scalarData': {'a2': {'fieldType': 'STRING', 'fields': [{'data': 'b2'}]}}, 'distance': 0.5491189}, {'id': 4, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.77582663, 0.8826412, 0.364886, 0.6153962, 0.07538124, 0.368824], 'binaryValues': []}, 'scalarData': {'a4': {'fieldType': 'STRING', 'fields': [{'data': 'b4'}]}}, 'distance': 0.9491992}]}
# RuntimeError

# index_name
# xq = [float1, float2, float3]
# topk
# data = {data1key:data1value}
# search_paras = []
b4 = x.vector_search("test", vectors[0], 10, {"meta_expr": {"a1": "b1"}})
print(b4) 
# b4 = {'vectorWithDistances': [{'id': 1, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.19151945, 0.62210876, 0.43772775, 0.7853586, 0.77997583, 0.2725926], 'binaryValues': []}, 'scalarData': {'a1': {'fieldType': 'STRING', 'fields': [{'data': 'b1'}]}}, 'distance': 0.0}]}
# RuntimeError

b5 = x.get_index()
print(b5)
# b5 = ['testResult2', 'test11', 'testResult1', 'testPart', 'testdingo2', 'test', 'hch_index1', 'hch_index', 'testPart2', 'hch_index2', 'testhch', 'hch_index3', 'testPart3', 'test_index', 'test_index21', 'testSearch1', 'hhh', 'test_index111', 'testSearch2', 'hchsss', 'test_index112']
# RuntimeError

# index_name
# ids = [id1, id2, id3]
b6 = x.vector_get("test", [1, 2, 3])
print(b6)
# b6 = [{'id': 1, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.19151945, 0.62210876, 0.43772775, 0.7853586, 0.77997583, 0.2725926], 'binaryValues': []}, 'scalarData': {'a1': {'fieldType': 'STRING', 'fields': [{'data': 'b1'}]}}}, {'id': 2, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.27746424, 0.8018722, 0.95813936, 0.87593263, 0.35781726, 0.5009951], 'binaryValues': []}, 'scalarData': {'a2': {'fieldType': 'STRING', 'fields': [{'data': 'b2'}]}}}, {'id': 3, 'vector': {'dimension': 6, 'valueType': 'FLOAT', 'floatValues': [0.6854629, 0.71270204, 0.37025076, 0.5611962, 0.50308317, 0.013768449], 'binaryValues': []}, 'scalarData': {'a3': {'fieldType': 'STRING', 'fields': [{'data': 'b3'}]}}}]
# RuntimeError

# index_name
b7 = x.get_max_index_row("test")
print(b7)
# b7 = id
# RuntimeError

# index_name
# ids = [id1, id2, id3]
b8 = x.vector_delete("test", [1, 2, 3])
print(b8)
# b8 = [True, True, True]
# RuntimeError

# index_name
# max_element
b9 = x.update_index_max_element("test", 50001)
print(b9)
# b9 = True
# RuntimeError
print(x.describe_index_info("test"))

# index_name
b10 = x.delete_index("test")
print(b10)
# b10 = True
# RuntimeError
